/*
 * post_raw.h
 * Main header file for libpost_raw
 * listpostal project
 * libpost_raw library
 * $Id: post_raw.h,v 1.66 2003/05/26 09:08:35 fullermd Exp $
 */

#ifndef _LIBPOSTAL_POST_RAW_H_
#define _LIBPOSTAL_POST_RAW_H_


/* Header files including header files.  Bad monkey! */
#include <stdio.h>
#include <sys/types.h>
#include <netinet/in.h>


/* Constants */
/* File lock types */
#define	LOCK_FLOCK		0x01
#define	LOCK_FLOCK_EX	0x02
#define	LOCK_DOTLOCK	0x04
#define	LOCK_FCNTL		0x08

#define LOCK_DEFAULT	(LOCK_DOTLOCK | LOCK_FCNTL)

#define	OPEN_READONLY	1
#define	OPEN_READWRITE	2

/* Position for writing/reading messages in a file mailbox */
#define	POS_CUR			1
#define POS_END			2
#define	POS_START		3

/* How much of a message to get (each level includes the previous) */
#define	GET_OFFSET		0
#define	GET_HEADER		1
#define	GET_BODY		2

/* Message statae */
#define STATUS_NULL		0x00
#define STATUS_NEW		0x01
#define STATUS_OLD		0x02
#define STATUS_READ		0x04
#define STATUS_REPLIED	0x08

/* What kind of source a message came from */
#define	SRC_NONE		0
#define	SRC_POP			1
#define	SRC_MBOX		2
#define	SRC_MDIR		3

/* Maildir location */
#define	LOC_TMP			1
#define	LOC_NEW			2
#define	LOC_CUR			3


/* Typedef */
typedef struct postal_conn			POSTAL_CONN;
typedef struct postal_msg			POSTAL_MSG;
typedef struct postal_msg_group		POSTAL_MSG_GROUP;

typedef struct spec_pop				SPEC_POP;
typedef struct pop_ret_stat			POP_RET_STAT;
typedef struct pop_ret_list			POP_RET_LIST;

typedef struct spec_mbox			SPEC_MBOX;

typedef struct spec_mdir			SPEC_MDIR;
typedef struct maildir_msg			MAILDIR_MSG;


/* Function prototypes */
/* Generic */
int postal_add_header(POSTAL_MSG *, const char *);
POSTAL_MSG *postal_alloc_msg(void);
int postal_change_header(POSTAL_MSG *, const char *, const char *);
int postal_del_header(POSTAL_MSG *, const char *);
void postal_free_msg(POSTAL_MSG *);
void postal_free_msggroup(POSTAL_MSG_GROUP *);
char *postal_get_header(const char *, const char *);
int postal_set_header(POSTAL_MSG *, const char *, const char *);

/* Maildir/ */
void maildir_close(POSTAL_CONN *);
int maildir_create(const char *);
int maildir_get_message(POSTAL_CONN *, POSTAL_MSG *, int);
long maildir_get_message_all(POSTAL_CONN *, int);
int maildir_get_status(POSTAL_MSG *);
POSTAL_CONN *maildir_open(const char *, int);
int maildir_write_msg(POSTAL_CONN *, POSTAL_MSG *);

/* mbox */
void mbox_close_file(POSTAL_CONN *);
int mbox_get_message(POSTAL_CONN *, POSTAL_MSG *, int);
long mbox_get_message_all(POSTAL_CONN *, int);
POSTAL_MSG *mbox_get_message_next(POSTAL_CONN *, int);
int mbox_get_status(POSTAL_MSG *);
POSTAL_CONN *mbox_open_file(const char *, int, int);
void mbox_write_msg(POSTAL_CONN *, POSTAL_MSG *, int);

/* POP */
int pop_close(POSTAL_CONN *);
POSTAL_CONN *pop_connect(const char *, const char *, const char *);
int pop_dele(POSTAL_CONN *, int);
void pop_free_retlist(POP_RET_LIST *);
POP_RET_LIST *pop_list(POSTAL_CONN *);
int pop_login(POSTAL_CONN *, const char *, const char *);
POSTAL_CONN *pop_network_connect(struct sockaddr_in);
POSTAL_MSG *pop_retr(POSTAL_CONN *, int);
POSTAL_MSG_GROUP *pop_retr_all(POSTAL_CONN *, int);
POP_RET_STAT *pop_stat(POSTAL_CONN *);


/* Macros */
#define PCONN_SIZE		(sizeof(POSTAL_CONN))
#define SPOP_SIZE		(sizeof(SPEC_POP))
#define SMBOX_SIZE		(sizeof(SPEC_MBOX))
#define SMDIR_SIZE		(sizeof(SPEC_MDIR))
#define PRSTAT_SIZE		(sizeof(POP_RET_STAT))
#define PRLIST_SIZE		(sizeof(POP_RET_LIST))
#define PMSG_SIZE		(sizeof(POSTAL_MSG))
#define PMSGGROUP_SIZE	(sizeof(POSTAL_MSG_GROUP))
#define MMSG_SIZE		(sizeof(MAILDIR_MSG))


/* Information access macros */
/* Generic */
#define postal_mgroup(pconn)		(pconn)->msgs
#define postal_msg_header(prs)		(prs)->header
#define postal_msg_body(prs)		(prs)->body
#define postal_msg_offset(prs)		(prs)->loc.offset
#define postal_msg_hsize(prs)		(prs)->hdrlen
#define postal_msg_bsize(prs)		(prs)->bodylen
#define postal_msg_mbstat(prs)		(prs)->stat_loc
#define postal_msg_newstat(prs)		(prs)->stat_new
#define postal_mgroup_num(prs)		(prs)->num_msgs
#define postal_mgroup_getmsg(prs,num) ((prs->msgs)[num-1])
#define postal_mlist_msg(prs)		(prs)->msg
#define postal_mlist_next(prs)		(prs)->next
/* POP */
#define pop_stat_msgs(prs)			(prs)->num_msg
#define pop_stat_bytes(prs)			(prs)->num_bytes
#define pop_list_mnum(prs)			(prs)->msg_num
#define pop_list_msize(prs)			(prs)->msg_size
#define pop_list_mnext(prs)			(prs)->next


/* Actiion macros */
/* mbox */
#define mbox_msg_seekto(msg, box)		( fseek(box->spec.mbox->file, \
											postal_msg_offset(msg), \
											SEEK_SET) )
#define mbox_msg_seekto_end(msg, box)	( fseek(box->spec.mbox->file, \
											( postal_msg_offset(msg) + \
												postal_msg_size(msg) ), \
											SEEK_SET) )


/* Structures */
/* postal_conn: A single libpost_raw connection */
struct postal_conn
{
	union
	{
		SPEC_POP	*pop;
		SPEC_MBOX	*mbox;
		SPEC_MDIR	*mdir;
	} spec;
	POSTAL_MSG_GROUP	*msgs;
};


/* spec_pop: Special info for POP connections */
struct spec_pop
{
	int		sock;
	char	*srv_header;		/* Server greeting header */
};


/* spec_mbox: Special info for mbox's */
struct spec_mbox
{
	char	*filename;			/* Filename */
	int		locks;				/* Lock types applied */
	FILE	*file;				/* stdio file stream */
	int		desc;				/* raw file descriptor */
};


/* spec_maildir: Special info for Maildir/'s */
struct spec_mdir
{
	char	*base;				/* Base directory */
};


/* pop_ret_stat: Information return from POP STAT query */
struct pop_ret_stat
{
	unsigned int	num_msg;
	unsigned long	num_bytes;
};


/* pop_ret_list: Individual 'list' message */
struct pop_ret_list
{
	unsigned int	msg_num;
	unsigned long	msg_size;
	POP_RET_LIST	*next;
};

/* postal_msg: Parsed out full email message */
struct postal_msg
{
	char			*header;
	size_t			hdrlen;
	char			*body;
	size_t			bodylen;
	union
	{
		MAILDIR_MSG	*file;
		off_t		offset;
	} loc;
	int				src_type;
	int				stat_loc;
	int				stat_new;
	int				dirty;
};

/* postal_msg_group: A group of parsed out messages */
struct postal_msg_group
{
	POSTAL_MSG	**msgs;
	long		num_msgs;
	long		alloced;
};

/* maildir_msg: A single message in a Maildir/ */
struct maildir_msg
{
	char		*filename;
	char		*unique;
	off_t		size;
	char		*info;
	int			loc;
	char		*obase;
};

#endif /* !_LIBPOSTAL_POST_RAW_H_ */
