/*
 * postal_dotlock.c
 * dotlock a file
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_dotlock.c,v 1.5 2003/05/26 09:43:09 fullermd Exp $
 */

/* sys/file.h needed on Linux for LOCK_* */
#include <sys/file.h>
#include <sys/param.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>

/*
 * These should be something other than global vars someday,
 * if we want to ever be threadsafe.
 */

unsigned dotlock_wait=1;
int dotlock_retry=5;

int
postal_dotlock(char *tolock, int type)
{
	int count;
	int toret;
	pid_t mypid;
	int randret;
	char tmpfile[MAXPATHLEN+1], *lockfile;
	FILE *tmpf;

	/* Where's the lockfile? */
	lockfile = malloc(strlen(tolock)+5+1);
	sprintf(lockfile, "%s.lock", tolock);

	/* Figure out what we're doing */
	if(type&LOCK_UN)
	{
		/* Unlocking is easy */
		toret = unlink(lockfile);
		free(lockfile);
		return(toret);
	}
	else
	{
		/* Locking is a bit more work */
		/* Build up some filenames */
		randret = rand();
		mypid = getpid();
		sprintf(tmpfile, "%s.lock.%d_%d", tolock, (int)mypid, randret);

		/* Open our tempfile */
		tmpf = fopen(tmpfile, "w");
		if(tmpf==NULL)
		{
			/* Something bad */
			return(-1);
		}

		/*
	 	 * Stick our PID in the lockfile.  This could be used later in the
	 	 * implementation to try and clear up stale lockfiles, but we won't
	 	 * do that for right now; among other things, it's not NFS clean.
	 	 * It's just...   well, something for housekeeping, I guess.
	 	 */
		fprintf(tmpf, "%d", (int)mypid);
		fflush(tmpf);
		fclose(tmpf);

		count=0;
		while(count < dotlock_retry)
		{
			/* Try and link our temporary file over to the real lockfile */
			if(link(tmpfile, lockfile)==0)
				break;

			/* Somebody else already has it.  Wait and retry */
			count++;
			sleep(dotlock_wait);
		}
		
		/* Clean up */
		unlink(tmpfile);
		free(lockfile);

		/* If we got here, dotlock succeeded */
		return(0);
	}

	/* Shouldn't get here */
	return(-1);
}
