/*
 * postal_list_dir.c
 * Return a list of files in a directory
 * libpostal project
 * libpost_raw gen code
 * $Id: postal_list_dir.c,v 1.8 2003/05/26 09:03:30 fullermd Exp $
 */

#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>


/* How many entries to alloc at once */
#define ALLOC_CHUNK		64


char **
postal_list_dir(const char *dir)
{
	char **toret=NULL;
	char *buf;
	DIR *mdir;
	struct dirent *aent;
	struct stat st_ret;
	long e_used, e_alloc;

	/* Check that it's really a dir */
	if( (stat(dir, &st_ret)) < 0 )
	{
		/* Something's screwed */
		return(NULL);
	}
	if( !(st_ret.st_mode & S_IFDIR) )
	{
		/* We're not a directory */
		return(NULL);
	}

	if( (mdir = opendir(dir)) == NULL )
	{
		/* Error opening the directory */
		return(NULL);
	}

	/* Now, start cycling through and adding to the list */
	e_used = e_alloc = 0;
	while( (aent = readdir(mdir)) != NULL )
	{
		/* Only do stuff if we're a regular file */
		if( (aent->d_type==DT_REG) )
		{
			/* First, alloc more memory in our list if we need to */
			/* XXX Really should malloc() and walk-copy, to prevent Bad Things */
			if( (e_used+1) > e_alloc)
			{
				void *tmploc;

				toret = realloc(toret,
						((sizeof(char *))*(e_alloc+ALLOC_CHUNK)) );
				if(toret==NULL)
				{
					/* Who knows? */
					closedir(mdir);
					return(NULL);
				}
				tmploc = toret + e_alloc;
				e_alloc += ALLOC_CHUNK;

				/* NULL 'em just to be safe */
				memset(tmploc, (int)NULL, (sizeof(char *) * ALLOC_CHUNK));
			}

			buf = strdup(aent->d_name);
			if(buf==NULL)
			{
				/* Bail */
				closedir(mdir);
				free(toret); /* XXX Need to walk this */
				return(NULL);
			}
			toret[e_used]=buf;
			e_used++;
		}
	}

	/* Got 'em all.  Do some memory trimming */
	/* XXX Really should malloc() and walk-copy, to prevent Bad Things */
	if(e_alloc>e_used)
	{
		toret = realloc( toret, ((sizeof(char **))*(e_used+1)) );
		if(toret==NULL)
		{
			/* Beats me... */
			closedir(mdir);
			return(NULL);
		}
		toret[e_used]=NULL;
	}

	/* Home, James... */
	closedir(mdir);
	return(toret);
}
