/*
 * maildir_get_message.c
 * Get a given message out of a Maildir/
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_get_message.c,v 1.9 2003/05/26 09:03:30 fullermd Exp $
 */

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_extra_p.h>


int
maildir_get_message(POSTAL_CONN *p_conn, POSTAL_MSG *msg, int get_type)
{
	char *rawmsg=NULL, *rawloc;
	char *buf;
	size_t msgallocsz=0;
	size_t frlen, frtot=0;
	char mdloc[4], path[PATH_MAX+1];
	FILE *msgfile;
	struct stat stret;

	/*
	 * Minor cheat.  We should never GET_OFFSET in Maildir/, being as
	 * that makes absolutely zero sense.  So we shouldn't ever have a
	 * GET_OFFSET request.  But if we do, just check if we already have
	 * the name or not.
	 */
	if(get_type==GET_OFFSET)
	{
		if(msg->src_type!=SRC_MDIR)
			return(-1);

		if(strlen(msg->loc.file->filename)>1)
			return(0);
		else
			return(-1);
	}

	/* Find and open the given message file */
	if(msg->loc.file->loc == LOC_TMP)
		sprintf(mdloc, "tmp");
	else if(msg->loc.file->loc == LOC_NEW)
		sprintf(mdloc, "new");
	else if(msg->loc.file->loc == LOC_CUR)
		sprintf(mdloc, "cur");
	sprintf(path, "%s/%s/%s", p_conn->spec.mdir->base,
			mdloc, msg->loc.file->filename);

	/* Open wide! */
	msgfile = fopen(path, "r");
	if(msgfile==NULL)
	{
		return(-1);
	}

	/*
	 * If we're just getting the headers, read in 4k at a time until we
	 * get 'em all.  4k is probably too much; 2k is probably too much; 1k
	 * is probably enough most of the time.  But 4k sounds like a
	 * reasonable first approximation to me.
	 */
	if(get_type==GET_HEADER)
	{
		while(1)
		{
			rawmsg = realloc(rawmsg, (msgallocsz+4096));
			if(rawmsg==NULL)
			{
				fclose(msgfile);
				return(-1);
			}
			msgallocsz += 4096;
			rawloc = rawmsg + frtot;

			frlen = fread(rawloc, 1, 4095, msgfile);
			if(frlen==0)
			{
				/* Something's badly wrong */
				free(rawmsg);
				fclose(msgfile);
				return(-1);
			}
			frtot += frlen;
			rawloc[frlen] = '\0';

			buf = strstr(rawmsg, "\n\n");
			if(buf!=NULL)
			{
				/* We've found the end */
				buf++;
				*buf='\0';

				/* Copy over and return */
				msg->hdrlen = strlen(rawmsg);
				msg->header = strdup(rawmsg);
				if(msg->header==NULL)
				{
					free(rawmsg);
					fclose(msgfile);
					return(-1);
				}
				msg->stat_loc = maildir_get_status(msg);
				
				free(rawmsg);
				fclose(msgfile);
				return(0);
			}
		}
	}
	else if(get_type==GET_BODY)
	{
		/* We can cheat beautifully here */
		if( (stat(path, &stret)) < 0)
		{
			/* Error */
			fclose(msgfile);
			return(-1);
		}

		/* Bad style!  Reusing one variable! */
		frtot = stret.st_size;

		rawmsg = malloc(frtot+1);
		frlen = fread(rawmsg, frtot, 1, msgfile);
		if(frlen < frtot)
		{
			/* This should never happen */
			free(rawmsg);
			fclose(msgfile);
			return(-1);
		}
		rawmsg[frlen] = '\0';

		buf = strstr(rawmsg, "\n\n");
		if(buf==NULL)
		{
			free(rawmsg);
			fclose(msgfile);
			return(-1);
		}

		buf++;
		buf='\0';
		buf++;

		msg->hdrlen = strlen(rawmsg);
		msg->header = malloc(msg->hdrlen+1);
		if(msg->header==NULL)
		{
			free(rawmsg);
			fclose(msgfile);
			return(-1);
		}
		strncpy(msg->header, rawmsg, msg->hdrlen);
		msg->header[(msg->hdrlen+1)] = '\0';
		msg->stat_loc = maildir_get_status(msg);

		msg->bodylen = strlen(buf);
		msg->body = malloc(msg->bodylen+1);
		if(msg->body==NULL)
		{
			free(rawmsg);
			fclose(msgfile);
			return(-1);
		}
		strncpy(msg->body, buf, msg->bodylen);
		msg->body[(msg->bodylen+1)] = '\0';

		free(rawmsg);
		fclose(msgfile);

		return(0);
	}

	/* We should never get here */
	free(rawmsg);
	fclose(msgfile);
	return(-1);
}
