/*
 * maildir_msg_filename_construct.c
 * Construct a Maildir message filename from its parts
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_msg_filename_construct.c,v 1.2 2003/05/25 06:25:24 fullermd Exp $
 */

#include <sys/types.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_maildir_p.h>
#include <post_raw_extra_p.h>


int
maildir_msg_filename_construct(MAILDIR_MSG *minfo, int newloc)
{
	size_t fnlen;
	off_t stmp;

	/* If we're going new -> new, there's not much to do */
	if(minfo->loc == LOC_NEW)
		if(newloc == LOC_NEW)
		{
			if(minfo->filename!=NULL)
				free(minfo->filename);
			minfo->filename = strdup(minfo->unique);
			return(0);
		}

	/* Else, start crunching */
	if(newloc == LOC_CUR)
	{
		if(minfo->filename!=NULL)
			free(minfo->filename);

		/* How big do we need? */
		/* <unique>:2,<info>\0 */
		fnlen = strlen(minfo->unique) + 3 + strlen(minfo->info) + 1;
		/* Now the message size */
		if( (stmp=minfo->size) != 0)
		{
			fnlen+=4; /* ",S=" + first digit */
			while( ((int)(stmp/=10))>0 )
				fnlen++;
		}

		minfo->filename = malloc(fnlen);

		if(minfo->size)
			sprintf(minfo->filename, "%s,S=%li:2,%s", minfo->unique,
					(long)minfo->size, minfo->info);
		else
			sprintf(minfo->filename, "%s:2,%s", minfo->unique, minfo->info);

		return(0);
	}

	/* We shouldn't get here.  Error-checking sure would be nice */
	return(1);
}
