/*
 * maildir_write_msg.c
 * Write a message into a Maildir
 * libpostal project
 * libpost_raw maildir code
 * $Id: maildir_write_msg.c,v 1.8 2003/05/26 09:53:56 fullermd Exp $
 */

#include <sys/param.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_maildir_p.h>
#include <post_raw_extra_p.h>


/* Maximum number of times to try finding a filename */
#define MAX_NAME_TRIES	5


int
maildir_write_msg(POSTAL_CONN *p_conn, POSTAL_MSG *msg)
{
	char *frhdr, *infostr, *buf;
	char *orig_fname;
	char oldpath[PATH_MAX+1], newpath[PATH_MAX+1], oldloc[4], newloc[4];
	struct stat stret, stret2;
	FILE *md_file;
	struct timeval curtime;
	pid_t mypid;
	int randret;
	char hostname[MAXHOSTNAMELEN+1]; /* +1 not strictly necessary */
	int ntries;
	int sameloc;

	/* We need to cure \r\n's if we have them... */
	buf = postal_whack_cr(msg->body);
	if(buf==NULL)
		return(-1);
	free(msg->body);
	msg->body = buf;

	buf = postal_whack_cr(msg->header);
	if(buf==NULL)
		return(-1);
	free(msg->header);
	msg->header = buf;

	/* Set the body length, and fiddle with headers */
	msg->bodylen = strlen(msg->body);

	/* We don't need Content-Length:, and save From_ */
	postal_del_header(msg, "Content-Length:");
	if(!strncmp(msg->header, "From ", 5))
	{
		/* We have a From_.  Save it */
		frhdr = postal_get_header(msg->header, "From");
		postal_set_header(msg, "X-From_:", frhdr);
		postal_del_header(msg, "From");
		free(frhdr);
		msg->hdrlen = strlen(msg->header);
	}

	/* Figure out where to write */
	/*
	 * First, if it's from a Maildir, check and see if it's from THIS
	 * Maildir.  The best way I can think of is comparing device/inode's
	 * for the root dirs, so that's what we're going to do.  This blows 2
	 * stat(2)'s for each message, so if you're moving bulk messages,
	 * clean up the ->loc's first!
	 */
	if( (msg->src_type==SRC_MDIR) )
	{
		stat(msg->loc.file->obase, &stret);
		stat(p_conn->spec.mdir->base, &stret2);

		/* If device and inode match, it's the same */
		if((stret.st_dev==stret2.st_dev)&&(stret.st_ino==stret2.st_ino))
			sameloc=1;
		else
			sameloc=0;
	}
	
	if( (msg->src_type==SRC_MDIR) && (sameloc==1) )
	{
		/* It already exists in this Maildir */
		/* Derive new information */
		if(msg->loc.file->info!=NULL)
			free(msg->loc.file->info);
		msg->loc.file->info = maildir_status2info(msg->stat_new);
		msg->loc.file->size = msg->hdrlen + 1 + msg->bodylen + 1;

		/* Now let's see */
		orig_fname = msg->loc.file->filename;
		msg->loc.file->filename=NULL;
		maildir_msg_filename_construct(msg->loc.file, msg->stat_new);

		/* Build up full pathnames */
		if(msg->loc.file->loc==LOC_NEW)
			sprintf(oldloc, "new");
		else if(msg->loc.file->loc==LOC_CUR)
			sprintf(oldloc, "cur");
		else if(msg->loc.file->loc==LOC_TMP)
			sprintf(oldloc, "tmp");

		if(msg->stat_new & STATUS_NEW)
			sprintf(newloc, "new");
		else
			sprintf(newloc, "cur");

		sprintf(oldpath, "%s/%s/%s", p_conn->spec.mdir->base,
				oldloc, orig_fname);
		sprintf(newpath, "%s/%s/%s", p_conn->spec.mdir->base,
				newloc, msg->loc.file->filename);

		free(orig_fname);

		/* If they don't match, we need to move */
		if( strcmp(oldpath, newpath)!=0 )
		{
			/*
			 * Gotta move!
			 * First, double-check that there's no file already at the
			 * place we're trying to move to.  If there is, panic.
			 */
			if( stat(newpath, &stret)==0 )
			{
				/* ACK! */
				return(-1); 
			}

			/* Move the puppy! */
			/* XXX Should we link() then unlink()?  I don't think so... */
			if(rename(oldpath, newpath)<0)
			{
				/* Double ACK! */
				return(-1);
			}


			/* It's done.  Update our info */
			if(msg->stat_new & STATUS_NEW)
				msg->loc.file->loc = LOC_NEW;
			else
				msg->loc.file->loc = LOC_CUR;

			msg->stat_loc = msg->stat_new;
		}

	}
	else
	{
		/* It's a new message to this Maildir.  It needs somewhere to live */
		msg->loc.file = malloc(MMSG_SIZE);
		msg->loc.file->unique = NULL;
		msg->loc.file->info = NULL;
		msg->loc.file->filename = NULL;

		/*
		 * Notes:
		 * We're going to be using the general naming schema that Courier
		 * uses for creating files in a Maildir.  It's a slightly
		 * extended form (though naturally fully compatible, since the
		 * "official" specification is pretty freeform) of the purist
		 * naming, but gives a solid width of uniqueness.
		 * See <http://www.courier-mta.org/maildir.html>
		 */

		/* Where should it go? */
		if(msg->stat_new & STATUS_NEW)
			msg->loc.file->loc = LOC_NEW;
		else
			msg->loc.file->loc = LOC_CUR;

rechoose:
		/* What's unique? */
		gettimeofday(&curtime, NULL);
		mypid = getpid();
		gethostname(hostname, MAXHOSTNAMELEN);

		/*
		 * I'm using a random number instead of an incrementing
		 * per-process counter, because the latter would be hard to
		 * implement since we're several layers deep inside a library.  I
		 * think that even without this, we'd have no real chance of
		 * collision, but I think our chances of having two messages
		 * delivered by the same process on the same host in the same
		 * microsecond with the same random number, with the messages the
		 * same size, are so low that nobody in their right mind should
		 * care.  When the message is linked into a permanent location,
		 * it'll have the device and inode also in the filename, which
		 * only has a chance of colliding on some very non-traditional
		 * filesystems.  If you want to be pedantic and go even further,
		 * then be my guest.  I can't be arsed.
		 */
		randret = rand();


		/* Re-use these variables */
		sprintf(oldpath, "%s/tmp/%li.M%liP%d_%d.%s",
				p_conn->spec.mdir->base, curtime.tv_sec,
				(long)curtime.tv_usec, mypid, randret, hostname);

		/* Test existence */
		if(stat(oldpath, &stret)==0)
		{
			/* Already exists.  This shouldn't happen */
			ntries++;
			if(ntries == MAX_NAME_TRIES)
			{
				free(msg->loc.file);
				msg->loc.file=NULL;
				return(-1);
			}
			sleep(2);
			goto rechoose;
		}

		/* Name is good.  Grab it */
		if( (md_file=fopen(oldpath, "w")) == NULL )
		{
			free(msg->loc.file);
			msg->loc.file=NULL;
			return(-1);
		}

		fprintf(md_file, "%s\n%s", msg->header, msg->body);
		fflush(md_file);
		fclose(md_file);

		/* Build up the new location */
		if(msg->loc.file->loc==LOC_CUR)
		{
			msg->loc.file->info = maildir_status2info(msg->stat_new);
			infostr = malloc(strlen(msg->loc.file->info)+4);
			sprintf(infostr, ":2,%s", msg->loc.file->info);
		}
		else
			infostr = NULL;
		stat(oldpath, &stret);
		sprintf(newpath, "%s/%s/%li.M%liP%dV%liI%li_%d.%s,S=%li%s",
				p_conn->spec.mdir->base,
				((msg->loc.file->loc==LOC_NEW)?"new":"cur"),
				curtime.tv_sec, (long)curtime.tv_usec, mypid,
				(long)stret.st_dev, (long)stret.st_ino, randret,
				hostname, (long)stret.st_size,
				((msg->loc.file->loc==LOC_CUR)?infostr:"") );
		free(infostr);

		/* We should handle this better */
		if( stat(newpath, &stret)==0 )
		{
			free(msg->loc.file);
			msg->loc.file=NULL;
			return(-1);
		}

		/* This could be done better too */
		if(rename(oldpath, newpath)<0)
		{
			free(msg->loc.file);
			msg->loc.file=NULL;
			return(-1);
		}


		/* Done and taken care of.  Bring our structure up to date */
		msg->loc.file->filename = strdup(newpath);
		maildir_msg_filename_dissect(msg->loc.file);

		/* We skip over the stuff below */
		msg->dirty=0;
	}


	/*
	 * If we get here, the file is wherever it needs to be.
	 * See if we have any data to write.
	 */
	if(msg->dirty)
	{
		/* We do.  Spew it. */
		if(msg->loc.file->loc == LOC_NEW)
			sprintf(newloc, "new");
		else if(msg->loc.file->loc == LOC_CUR)
			sprintf(newloc, "cur");
		else if(msg->loc.file->loc == LOC_TMP)
			sprintf(newloc, "tmp");

		sprintf(newpath, "%s/%s/%s", p_conn->spec.mdir->base,
				newloc, msg->loc.file->filename);

		if( (md_file=fopen(newpath, "w")) == NULL )
			return(-1);

		fprintf(md_file, "%s\n%s", msg->header, msg->body);
		fflush(md_file);
		fclose(md_file);
	}

	/* Fin */
	return(0);
}
