/*
 * mbox_open_file.c
 * Open a file to use as a mbox
 * libpostal project
 * libpost_raw mbox code
 * $Id: mbox_open_file.c,v 1.19 2003/05/26 09:03:31 fullermd Exp $
 */

#include <sys/file.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_mbox_p.h>
#include <post_raw_extra_p.h>


POSTAL_CONN *
mbox_open_file(const char *fname, int locktype, int readwrite)
{
	POSTAL_CONN *toret;
	struct stat st_ret;

	/* Allocate memory */
	toret = malloc(PCONN_SIZE);
	toret->spec.mbox = malloc(SMBOX_SIZE);
	toret->msgs = NULL;

	/* Copy the filename into the SPEC_ */
	toret->spec.mbox->filename = malloc( (strlen(fname)+1) );
	strcpy(toret->spec.mbox->filename, fname);

	/*
	 * Open the file as a raw I/O stream, lock it, and get a
	 * stdio stream, since it's more fun to manipulate.
	 * Check READONLY or READWRITE
	 */
	if( (readwrite == OPEN_READONLY) )
	{
		if((toret->spec.mbox->desc = open(fname, O_RDONLY))<0)
		{
			free(toret->spec.mbox);
			free(toret);
			return(NULL);
		}
		toret->spec.mbox->file = fdopen(toret->spec.mbox->desc, "r");
		if(mbox_lock_read(toret, locktype)!=0)
		{
			fclose(toret->spec.mbox->file);
			mbox_unlock(toret);
			free(toret->spec.mbox->filename);
			free(toret->spec.mbox);
			free(toret);
			return(NULL);
		}
	}
	else /* OPEN_READWRITE is the only current case */
	{
		if((toret->spec.mbox->desc = open(fname, O_RDWR | O_CREAT))<0)
		{
			free(toret->spec.mbox);
			free(toret);
			return(NULL);
		}
		toret->spec.mbox->file = fdopen(toret->spec.mbox->desc, "r+");
		if(mbox_lock_write(toret, locktype)!=0)
		{
			fclose(toret->spec.mbox->file);
			mbox_unlock(toret);
			free(toret->spec.mbox->filename);
			free(toret->spec.mbox);
			free(toret);
			return(NULL);
		}
	}

	/*
	 * Set the permissions in a portable way.
	 * XXX Should we really hardcode this?
	 */
	fchmod(toret->spec.mbox->desc, S_IRUSR | S_IWUSR);

	/* Get the stat info */
	fstat(toret->spec.mbox->desc, &st_ret);

	/* Return the structure, we're ready to rock */
	return(toret);
}
