/*
 * pop_retr.c
 * POP 'RETR' query
 * libpostal project
 * libpost_raw  POP code
 * $Id: pop_retr.c,v 1.12 2003/05/26 09:03:31 fullermd Exp $
 */

#include <sys/types.h>
#include <sys/uio.h>
#include <limits.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include <post_raw.h>
#include <post_raw_p.h>
#include <post_raw_pop_p.h>
#include <post_raw_extra_p.h>


POSTAL_MSG *
pop_retr(POSTAL_CONN *p_conn, int msgnum)
{
	char truebuf[BUFFER_SIZE+1];
	char *inmsg=NULL, *buf, *buf2;
	int moretoget=1, firsttime=1;
	ssize_t readlen;
	size_t msize;
	POSTAL_MSG *toret;

	/* Send request */
	sprintf(truebuf, "RETR %d\r\n", msgnum);
	POP_CMD_SEND(p_conn, truebuf);
	CLEARSTRING(truebuf);

	/* Start processing */
	while(moretoget)
	{
		/* Get the response */
		readlen=read(p_conn->spec.pop->sock, truebuf, BUFFER_SIZE);
		if(readlen<0)
			return(NULL);
		truebuf[readlen]='\0';
		buf=truebuf;

		/*
		 * If it's the first time through, make sure we're +OK.
		 * If we're not, we're in trouble, bail.
		 * XXX We really need a good error-handling system
		 */
		if(firsttime)
		{
			/* First, check that we have a good response, else bail */
			if( (strncmp(buf, POP_GOOD, strlen(POP_GOOD)) != 0) )
			{
				if(inmsg)
					free(inmsg);
				return(NULL);
			}
			buf2 = strchr(buf, (int) '\r');
			buf2+=2;
			buf = buf2;
			firsttime=0;
		}

		/* Store up the whole thing */
		msize = strlen(buf)+1;
		if(inmsg)
		{
			msize += strlen(inmsg);
			inmsg = realloc(toret, msize);
		}
		else
		{
			inmsg = malloc(msize);
			*inmsg = '\0';	/* This shouldn't be necessary.  Sigh. */
		}
		strcat(inmsg, buf);

		/*
		 * Check to see if we've got the 'end' of the return.
		 */
		buf2 = strrchr(inmsg, (int) '.');
		if(buf2)
		{
			if( (strncmp((buf2-2), "\r\n.\r\n", 5) == 0) )
			{
				/* We have it all now */
				moretoget=0;
				*(buf2-2) = '\0';
			}
		}
	} /* Got it all */

	/*
	 * Parse out into a POSTAL_MSG struct.
	 * First, seperate out the headers, then the body, and plop those
	 * puppies in.
	 */
	buf = strstr(inmsg, "\r\n\r\n");
	if(!buf)
	{
		/* Error handling would be nice */
		free(inmsg);
		return(NULL);
	}
	buf+=2;
	*buf='\0';
	buf+=2;

	/* Malloc me a hillock */
	toret = postal_alloc_msg();
	toret->src_type = SRC_POP;

	/* We can reuse this var now */
	msize = strlen(inmsg);
	toret->header = malloc(msize+1);
	strcpy(toret->header, inmsg);
	toret->hdrlen = msize;
	msize = strlen(buf);
	toret->body = malloc(msize+1);
	strcpy(toret->body, buf);
	toret->bodylen = msize;

	/*
	 * Unfortunately, I can't think of a good way of finding status on
	 * messages retrieves from a POP server.  I suppose that if they're
	 * in a mbox, they'll have a Status or X-Status: header, but if
	 * they're not, they won't, and if they're new, they won't, so who
	 * the heck knows?
	 * For the sake of sanity, we'll define (not assume, define! :) that
	 * all messages retrieved through POP are 'new'.
	 */
	toret->stat_new = STATUS_NEW;

	/* Clean up and return */
	free(inmsg);
	return(toret);
}
